@@ -7,19 +7,24 @@ use App::Ack ();
 use App::Ack::ConfigDefault ();
 use App::Ack::ConfigFinder ();
 use App::Ack::Filter;
+use App::Ack::Filter::Collection;
 use App::Ack::Filter::Default;
 use Carp 1.04 ();
-use Getopt::Long 2.35 ();
+use Getopt::Long 2.38 ();
 use Text::ParseWords 3.1 ();
 
-=head1 App::Ack::ConfigLoader
+=head1 NAME
+
+App::Ack::ConfigLoader
+
+=head1 DESCRIPTION
+
+Logic for loading configuration files.
 
 =head1 FUNCTIONS
 
 =head2 process_args( @sources )
 
-
-
 =cut
 
 my @INVALID_COMBINATIONS;
@@ -49,6 +54,58 @@ BEGIN {
     );
 }
 
+sub _generate_ignore_dir {
+    my ( $option_name, $opt ) = @_;
+
+    my $is_inverted = $option_name =~ /^--no/;
+
+    return sub {
+        my ( undef, $dir ) = @_;
+
+        $dir = App::Ack::remove_dir_sep( $dir );
+        if ( $dir !~ /:/ ) {
+            $dir = 'is:' . $dir;
+        }
+
+        my ( $filter_type, $args ) = split /:/, $dir, 2;
+
+        if ( $filter_type eq 'firstlinematch' ) {
+            Carp::croak( qq{Invalid filter specification "$filter_type" for option '$option_name'} );
+        }
+
+        my $filter = App::Ack::Filter->create_filter($filter_type, split(/,/, $args));
+        my $collection;
+
+        my $previous_inversion_matches = $opt->{idirs} && !($is_inverted xor $opt->{idirs}[-1]->is_inverted());
+
+        if ( $previous_inversion_matches ) {
+            $collection = $opt->{idirs}[-1];
+
+            if ( $is_inverted ) {
+                # XXX this relies on invert of an inverted filter
+                #     to return the original
+                $collection = $collection->invert()
+            }
+        }
+        else {
+            $collection = App::Ack::Filter::Collection->new();
+
+            if ( $is_inverted ) {
+                push @{ $opt->{idirs} }, $collection->invert();
+            }
+            else {
+                push @{ $opt->{idirs} }, $collection;
+            }
+        }
+
+        $collection->add($filter);
+
+        if ( $filter_type eq 'is' ) {
+            $collection->add(App::Ack::Filter::IsPath->new($args));
+        }
+    };
+}
+
 sub process_filter_spec {
     my ( $spec ) = @_;
 
@@ -73,6 +130,7 @@ sub process_filter_spec {
     }
 }
 
+
 sub uninvert_filter {
     my ( $opt, @filters ) = @_;
 
@@ -90,8 +148,11 @@ sub uninvert_filter {
             $i--;
         }
     }
+
+    return;
 }
 
+
 sub process_filetypes {
     my ( $opt, $arg_sources ) = @_;
 
@@ -183,27 +244,32 @@ sub process_filetypes {
     return \%additional_specs;
 }
 
+
 sub removed_option {
     my ( $option, $explanation ) = @_;
 
     $explanation ||= '';
     return sub {
-        warn "Option '$option' is not valid in ack 2\n$explanation";
+        warn "Option '$option' is not valid in ack 2.\n$explanation";
         exit 1;
     };
 }
 
+
 sub get_arg_spec {
     my ( $opt, $extra_specs ) = @_;
 
-    my $dash_a_explanation = <<EOT;
-This is because we now have -k/--known-types which makes it only select files
-of known types, rather than any text file (which is the behavior of ack 1.x).
-You may have options in a .ackrc, or in the ACKRC_OPTIONS environment variable.
-Try using the --dump flag.
+    my $dash_a_explanation = <<'EOT';
+You don't need -a, ack 1.x users.  This is because ack 2.x has
+-k/--known-types which makes it only select files of known types, rather
+than any text file (which is the behavior of ack 1.x).
+
+If you're surprised to see this message because you didn't put -a on the
+command line, you may have options in an .ackrc, or in the ACKRC_OPTIONS
+environment variable.  Try using the --dump flag to help find it.
 EOT
 
-=for Adding-Options
+=begin Adding-Options
 
     *** IF YOU ARE MODIFYING ACK PLEASE READ THIS ***
 
@@ -216,16 +282,28 @@ EOT
     * Your new option is explained when a user invokes ack --man.
       (See the POD at the end of ./ack)
     * Add your option to t/config-loader.t
+    * Add your option to t/Util.pm#get_options
+    * Add your option's description and aliases to dev/generate-completion-scripts.pl
     * Go through the list of options already available, and consider
       whether your new option can be considered mutually exclusive
       with another option.
+
+=end Adding-Options
+
 =cut
+
+    sub _context_value {
+        my $val = shift;
+
+        # Contexts default to 2.
+        return (!defined($val) || ($val < 0)) ? 2 : $val;
+    }
+
     return {
         1                   => sub { $opt->{1} = $opt->{m} = 1 },
-        'A|after-context=i' => \$opt->{after_context},
-        'B|before-context=i'
-                            => \$opt->{before_context},
-        'C|context:i'       => sub { shift; my $val = shift; $opt->{before_context} = $opt->{after_context} = ($val || 2) },
+        'A|after-context:-1'  => sub { shift; $opt->{after_context}  = _context_value(shift) },
+        'B|before-context:-1' => sub { shift; $opt->{before_context} = _context_value(shift) },
+        'C|context:-1'        => sub { shift; $opt->{before_context} = $opt->{after_context} = _context_value(shift) },
         'a'                 => removed_option('-a', $dash_a_explanation),
         'all'               => removed_option('--all', $dash_a_explanation),
         'break!'            => \$opt->{break},
@@ -256,19 +334,18 @@ EOT
         'h|no-filename'     => \$opt->{h},
         'H|with-filename'   => \$opt->{H},
         'i|ignore-case'     => \$opt->{i},
-        'ignore-directory|ignore-dir=s' # XXX Combine this version with the negated version below
-                            => sub {
-                                my ( undef, $dir ) = @_;
-
-                                $dir = App::Ack::remove_dir_sep( $dir );
-                                if ( $dir !~ /^(?:is|match):/ ) {
-                                    $dir = 'is:' . $dir;
-                                }
-                                push @{ $opt->{idirs} }, $dir;
-                               },
-        'ignore-file=s'    => sub {
+        'ignore-directory|ignore-dir=s' => _generate_ignore_dir('--ignore-dir', $opt),
+        'ignore-file=s'     => sub {
                                     my ( undef, $file ) = @_;
-                                    push @{ $opt->{ifiles} }, $file;
+
+                                    my ( $filter_type, $args ) = split /:/, $file, 2;
+
+                                    my $filter = App::Ack::Filter->create_filter($filter_type, split(/,/, $args));
+
+                                    if ( !$opt->{ifiles} ) {
+                                        $opt->{ifiles} = App::Ack::Filter::Collection->new();
+                                    }
+                                    $opt->{ifiles}->add($filter);
                                },
         'lines=s'           => sub { shift; my $val = shift; push @{$opt->{lines}}, $val },
         'l|files-with-matches'
@@ -285,25 +362,7 @@ EOT
 
             $opt->{pager} = $value || $ENV{PAGER};
         },
-        'noignore-directory|noignore-dir=s'
-                            => sub {
-                                my ( undef, $dir ) = @_;
-
-                                # XXX can you do --noignore-dir=match,...?
-                                $dir = App::Ack::remove_dir_sep( $dir );
-                                if ( $dir !~ /^(?:is|match):/ ) {
-                                    $dir = 'is:' . $dir;
-                                }
-                                if ( $dir !~ /^(?:is|match):/ ) {
-                                    Carp::croak("invalid noignore-directory argument: '$dir'");
-                                }
-
-                                @{ $opt->{idirs} } = grep {
-                                    $_ ne $dir
-                                } @{ $opt->{idirs} };
-
-                                push @{ $opt->{no_ignore_dirs} }, $dir;
-                               },
+        'noignore-directory|noignore-dir=s' => _generate_ignore_dir('--noignore-dir', $opt),
         'nopager'           => sub { $opt->{pager} = undef },
         'passthru'          => \$opt->{passthru},
         'print0'            => \$opt->{print0},
@@ -344,10 +403,12 @@ EOT
     }; # arg_specs
 }
 
+
 sub process_other {
     my ( $opt, $extra_specs, $arg_sources ) = @_;
 
-    Getopt::Long::Configure('default', 'no_auto_help', 'no_auto_version'); # start with default options, minus some annoying ones
+    # Start with default options, minus some annoying ones.
+    Getopt::Long::Configure('default', 'no_auto_help', 'no_auto_version');
     Getopt::Long::Configure(
         'bundling',
         'no_ignore_case',
@@ -387,17 +448,14 @@ sub process_other {
 
         if ( $source->{project} ) {
             my $illegal = sub {
-                my ( $option ) = @_;
-
-                return sub {
-                    die "Option $option is illegal in project ackrcs";
-                };
+                die "Options --output, --pager and --match are forbidden in project .ackrc files.\n";
             };
 
-            $args_for_source = { %$args_for_source,
-                'output=s'=> $illegal->('--output'),
-                'pager:s' => $illegal->('--pager'),
-                'match=s' => $illegal->('--match'),
+            $args_for_source = {
+                %{$args_for_source},
+                'output=s' => $illegal,
+                'pager:s'  => $illegal,
+                'match=s'  => $illegal,
             };
         }
 
@@ -427,10 +485,10 @@ sub process_other {
     return;
 }
 
+
 sub should_dump_options {
     my ( $sources ) = @_;
 
-
     foreach my $source (@{$sources}) {
         my ( $name, $options ) = @{$source}{qw/name contents/};
 
@@ -448,6 +506,7 @@ sub should_dump_options {
     return;
 }
 
+
 sub explode_sources {
     my ( $sources ) = @_;
 
@@ -483,7 +542,8 @@ sub explode_sources {
             $source->{contents} = $options =
                 [ Text::ParseWords::shellwords($options) ];
         }
-        for ( my $j = 0; $j < @{$options}; $j++ ) {
+
+        for my $j ( 0 .. @{$options}-1 ) {
             next unless $options->[$j] =~ /^-/;
             my @chunk = ( $options->[$j] );
             push @chunk, $options->[$j] while ++$j < @{$options} && $options->[$j] !~ /^-/;
@@ -508,6 +568,7 @@ sub explode_sources {
     return \@new_sources;
 }
 
+
 sub compare_opts {
     my ( $a, $b ) = @_;
 
@@ -520,6 +581,7 @@ sub compare_opts {
     return $first_a cmp $first_b;
 }
 
+
 sub dump_options {
     my ( $sources ) = @_;
 
@@ -548,12 +610,13 @@ sub dump_options {
     return;
 }
 
+
 sub remove_default_options_if_needed {
     my ( $sources ) = @_;
 
     my $default_index;
 
-    foreach my $index ( 0 .. $#$sources ) {
+    foreach my $index ( 0 .. $#{$sources} ) {
         if ( $sources->[$index]{'name'} eq 'Defaults' ) {
             $default_index = $index;
             last;
@@ -572,7 +635,7 @@ sub remove_default_options_if_needed {
         'pass_through',
     );
 
-    foreach my $index ( $default_index + 1 .. $#$sources ) {
+    foreach my $index ( $default_index + 1 .. $#{$sources} ) {
         my ( $name, $args ) = @{$sources->[$index]}{qw/name contents/};
 
         if (ref($args)) {
@@ -599,6 +662,7 @@ sub remove_default_options_if_needed {
     return \@copy;
 }
 
+
 sub check_for_mutually_exclusive_options {
     my ( $arg_sources ) = @_;
 
@@ -647,8 +711,11 @@ sub check_for_mutually_exclusive_options {
             }
         }
     }
+
+    return;
 }
 
+
 sub process_args {
     my $arg_sources = \@_;
 
@@ -731,7 +798,7 @@ sub retrieve_arg_sources {
 
     push @arg_sources, {
         name     => 'Defaults',
-        contents => [ App::Ack::ConfigDefault::options() ],
+        contents => [ App::Ack::ConfigDefault::options_clean() ],
     };
 
     foreach my $file ( @files) {
